Package: gemtc
Version: 1.1-1
Date: 2026-03-25
Title: Network Meta-Analysis Using Bayesian Methods
Author: Gert van Valkenhoef [aut, cre],
  Joel Kuiper [aut]
Authors@R: c(person(given = "Gert",
                    family = "van Valkenhoef",
                    role = c("aut", "cre"),
                    email = "gert@gertvv.nl"),
             person(given = "Joel",
                    family = "Kuiper",
                    role = "aut"))
Maintainer: Gert van Valkenhoef <gert@gertvv.nl>
Description: Network meta-analyses (mixed treatment comparisons) in the Bayesian
    framework using JAGS. Includes methods to assess heterogeneity and
    inconsistency, and a number of standard visualizations.
    van Valkenhoef et al. (2012) <doi:10.1002/jrsm.1054>;
    van Valkenhoef et al. (2015) <doi:10.1002/jrsm.1167>.
Depends: R (>= 3.5.0), coda (>= 0.13)
Imports: igraph (>= 1.5), meta (>= 2.1), plyr (>= 1.8), graphics,
        grDevices, stats, utils, grid, rjags (>= 4-0), truncnorm,
        Rglpk, forcats (>= 0.5.0)
Suggests: testthat (>= 0.8), Matrix
URL: https://github.com/gertvv/gemtc
License: GPL-3
LazyData: true
Collate: 'anohe.R' 'arrayize.R' 'blobbogram.R' 'template.R' 'code.R'
        'data.R' 'deviance.R' 'forest.R' 'solveLP.R' 'inits.R'
        'likelihoods.R' 'll-helper.counts.R' 'll.binom.cloglog.R'
        'll.binom.log.R' 'll.binom.logit.R' 'll.call.R'
        'll.normal.identity.R' 'll.normal.smd.R' 'll.poisson.log.R'
        'minimum.diameter.spanning.tree.R' 'mtc.data.studyrow.R'
        'mtc.hy.prior.R' 'mtc.model.R' 'mtc.model.consistency.R'
        'mtc.model.nodesplit.R' 'mtc.model.regression.R'
        'mtc.model.ume.R' 'mtc.model.use.R' 'mtc.network.R'
        'stopIfNotConsistent.R' 'mtc.result.R' 'mtc.run.R'
        'nodesplit.R' 'plotCovariateEffect.R' 'priors.R'
        'rank.probability.R' 'regression.R' 'relative.effect.R'
        'relative.effect.table.R'
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2026-03-25 19:50:07 UTC; gert
Repository: CRAN
Date/Publication: 2026-03-26 13:10:14 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 18:14:40 UTC; unix
Archs: gemtc.so.dSYM
